@@ -112,12 +112,7 @@ struct derived_probe: public probe
   derived_probe (probe* b);
   derived_probe (probe* b, probe_point* l);
   probe* base; // the original parsed probe
-
-private:
-  static unsigned last_probeidx;
-
-public:
-  std::string name;
+  virtual probe* basest () { return base->basest(); }
 
   virtual ~derived_probe () {}
 
@@ -145,6 +140,10 @@ public:
 protected:
   void emit_probe_prologue (translator_output* o, const std::string&);
   void emit_probe_epilogue (translator_output* o);
+
+public:
+  static void emit_common_header (translator_output* o);
+  // from c_unparser::emit_common_header
 };
 
 // ------------------------------------------------------------------------
@@ -179,6 +178,7 @@ match_key
   match_key & with_string();
   std::string str() const;
   bool operator<(match_key const & other) const;
+  bool globmatch(match_key const & other) const;
 };
 
 
@@ -211,7 +211,7 @@ match_node
 int semantic_pass (systemtap_session& s);
 void derive_probes (systemtap_session& s,
                     probe *p, std::vector<derived_probe*>& dps,
-                    bool exc_outermost = true);
+                    bool optional = false);
 
 // A helper we use here and in translate, for pulling symbols out of lvalue
 // expressions.
