@@ -46,7 +46,7 @@ version ()
 }
 
 void
-usage (systemtap_session& s)
+usage (systemtap_session& s, int exitcode)
 {
   version ();
   clog
@@ -67,6 +67,7 @@ usage (systemtap_session& s)
     << "   -u         unoptimized translation" << (s.unoptimized ? " [set]" : "") << endl
     << "   -g         guru mode" << (s.guru_mode ? " [set]" : "") << endl
     << "   -b         bulk (relayfs) mode" << (s.bulk_mode ? " [set]" : "") << endl
+    << "   -M         Don't merge per-cpu files for bulk (relayfs) mode" << (s.merge ? "" : " [set]") << endl
     << "   -s NUM     buffer size in megabytes, instead of "
     << s.buffer_size << endl
     << "   -p NUM     stop after pass NUM 1-5, instead of "
@@ -95,7 +96,7 @@ usage (systemtap_session& s)
     ;
   // -d: dump safety-related external references
 
-  exit (0);
+  exit (exitcode);
 }
 
 
@@ -136,12 +137,19 @@ main (int argc, char * const argv [])
   s.keep_tmpdir = false;
   s.cmd = "";
   s.target_pid = 0;
+  s.merge=true;
 
   const char* s_p = getenv ("SYSTEMTAP_TAPSET");
-  if (s_p != NULL)
+  if (s_p != NULL)  
+  {
     s.include_path.push_back (s_p);
+    s.include_path.push_back (string(s_p) + "/LKET");
+  }
   else
+  {
     s.include_path.push_back (string(PKGDATADIR) + "/tapset");
+    s.include_path.push_back (string(PKGDATADIR) + "/tapset/LKET");
+  }
 
   const char* s_r = getenv ("SYSTEMTAP_RUNTIME");
   if (s_r != NULL)
@@ -151,7 +159,7 @@ main (int argc, char * const argv [])
 
   while (true)
     {
-      int grc = getopt (argc, argv, "hVvtp:I:e:o:R:r:m:kgc:x:D:bs:u");
+      int grc = getopt (argc, argv, "hVMvtp:I:e:o:R:r:m:kgc:x:D:bs:u");
       if (grc < 0)
         break;
       switch (grc)
@@ -160,6 +168,10 @@ main (int argc, char * const argv [])
           version ();
           exit (0);
 
+        case 'M':
+          s.merge = false;
+          break;
+
         case 'v':
 	  s.verbose ++;
 	  break;
@@ -172,8 +184,8 @@ main (int argc, char * const argv [])
           s.last_pass = atoi (optarg);
           if (s.last_pass < 1 || s.last_pass > 5)
             {
-              cerr << "Invalid pass number." << endl;
-              usage (s);
+              cerr << "Invalid pass number (should be 1-5)." << endl;
+              usage (s, 1);
             }
           break;
 
@@ -183,7 +195,11 @@ main (int argc, char * const argv [])
 
         case 'e':
 	  if (have_script)
-	    usage (s);
+	    {
+	      cerr << "Only one script can be given on the command line."
+		   << endl;
+	      usage (s, 1);
+	    }
           cmdline_script = string (optarg);
           have_script = true;
           break;
@@ -224,8 +240,8 @@ main (int argc, char * const argv [])
           s.buffer_size = atoi (optarg);
           if (s.buffer_size < 1 || s.buffer_size > 64)
             {
-              cerr << "Invalid buffer size." << endl;
-              usage (s);
+              cerr << "Invalid buffer size (should be 1-64)." << endl;
+	      usage (s, 1);
             }
           break;
 
@@ -242,11 +258,27 @@ main (int argc, char * const argv [])
 	  break;
 
         case 'h':
+          usage (s, 0);
+          break;
+
         default:
-          usage (s);
+          usage (s, 1);
+          break;
         }
     }
 
+  if(!s.bulk_mode && !s.merge)
+    {
+      cerr << "-M option is valid only for bulk (relayfs) mode." <<endl;
+      usage (s, 1);
+    }
+
+  if(!s.output_file.empty() && s.bulk_mode && !s.merge)
+    {
+      cerr << "You can't specify -M, -b and -o options together." <<endl;
+      usage (s, 1);
+    }
+
   for (int i = optind; i < argc; i++)
     {
       if (! have_script)
@@ -260,7 +292,10 @@ main (int argc, char * const argv [])
 
   // need a user file
   if (! have_script)
-    usage(s);
+    {
+      cerr << "A script must be specified." << endl;
+      usage(s, 1);
+    }
 
   int rc = 0;
 
