@@ -30,14 +30,14 @@ parser::parser (systemtap_session& s, istream& i, bool p):
   session (s),
   input_name ("<input>"), free_input (0),
   input (i, input_name, s), privileged (p),
-  last_t (0), next_t (0), num_errors (0)
+  context(con_unknown), last_t (0), next_t (0), num_errors (0)
 { }
 
 parser::parser (systemtap_session& s, const string& fn, bool p):
   session (s),
   input_name (fn), free_input (new ifstream (input_name.c_str(), ios::in)),
   input (* free_input, input_name, s), privileged (p),
-  last_t (0), next_t (0), num_errors (0)
+  context(con_unknown), last_t (0), next_t (0), num_errors (0)
 { }
 
 parser::~parser()
@@ -72,16 +72,27 @@ tt2str(token_type tt)
     case tok_string: return "string";
     case tok_number: return "number";
     case tok_embedded: return "embedded-code";
+    case tok_keyword: return "keyword";
     }
   return "unknown token";
 }
 
+ostream&
+operator << (ostream& o, const source_loc& loc)
+{
+  o << loc.file << ":" 
+    << loc.line << ":"
+    << loc.column;
+
+  return o;
+}
+
 ostream&
 operator << (ostream& o, const token& t)
 {
   o << tt2str(t.type);
 
-  if (t.type != tok_embedded) // XXX: other types?
+  if (t.type != tok_embedded && t.type != tok_keyword) // XXX: other types?
     {
       o << " '";
       for (unsigned i=0; i<t.content.length(); i++)
@@ -93,9 +104,7 @@ operator << (ostream& o, const token& t)
     }
 
   o << " at " 
-    << t.location.file << ":" 
-    << t.location.line << ":"
-    << t.location.column;
+    << t.location;
 
   return o;
 }
@@ -244,10 +253,14 @@ parser::scan_pp ()
       // passing that as a vector to an evaluator.
 
       bool result = eval_pp_conditional (session, l, op, r);
+      delete l;
+      delete op;
+      delete r;
       
       const token *m = input.scan (); // NB: not recursive
       if (! (m && m->type == tok_operator && m->content == "%?"))
         throw parse_error ("expected '%?' marker for conditional", t);
+      delete m; // "%?"
 
       vector<const token*> my_enqueued_pp;
       
@@ -263,23 +276,32 @@ parser::scan_pp ()
           // enqueue token
           if (result) 
             my_enqueued_pp.push_back (m);
+          else
+            delete m; // unused token
           // continue
         }
       
       if (m && m->type == tok_operator && m->content == "%:") // ELSE
-        while (true)
-          {
-            m = scan_pp (); // NB: recursive
-            if (m == 0)
-              throw parse_error ("missing ELSE tokens for conditional", t);
-            
-            if (m->type == tok_operator && m->content == "%)") // END
-              break;
-            // enqueue token
-            if (! result) 
-              my_enqueued_pp.push_back (m);
-            // continue
-          }
+        {
+          delete m; // "%:"
+          while (true)
+            {
+              m = scan_pp (); // NB: recursive
+              if (m == 0)
+                throw parse_error ("missing ELSE tokens for conditional", t);
+              
+              if (m->type == tok_operator && m->content == "%)") // END
+                break;
+              // enqueue token
+              if (! result) 
+                my_enqueued_pp.push_back (m);
+              else
+                delete m; // unused token
+              // continue
+            }
+        }
+      delete t; // "%("
+      delete m; // "%)"
 
       // NB: we transcribe the retained tokens here, and not inside
       // the THEN/ELSE while loops.  If it were done there, each loop
@@ -352,6 +374,17 @@ parser::expect_unknown (token_type tt, string & target)
 }
 
 
+const token* 
+parser::expect_unknown2 (token_type tt1, token_type tt2, string & target)
+{
+  const token *t = next();
+  if (!(t && (t->type == tt1 || t->type == tt2)))
+    throw parse_error ("expected " + tt2str(tt1) + " or " + tt2str(tt2));
+  target = t->content;
+  return t;
+}
+
+
 const token* 
 parser::expect_op (std::string const & expected)
 {
@@ -383,6 +416,13 @@ parser::expect_ident (std::string & target)
 }
 
 
+const token* 
+parser::expect_ident_or_keyword (std::string & target)
+{
+  return expect_unknown2 (tok_identifier, tok_keyword, target);
+}
+
+
 bool 
 parser::peek_op (std::string const & op)
 {
@@ -497,6 +537,26 @@ lexer::scan ()
               n->content = arg;
             }
         }
+      else
+        {
+	  if (n->content    == "probe"
+	      || n->content == "global"
+	      || n->content == "function"
+	      || n->content == "if"
+	      || n->content == "else"
+	      || n->content == "for"
+	      || n->content == "foreach"
+	      || n->content == "in"
+	      || n->content == "return"
+	      || n->content == "delete"
+	      || n->content == "while"
+	      || n->content == "break"
+	      || n->content == "continue"
+	      || n->content == "next"
+	      || n->content == "string"
+	      || n->content == "long")
+	    n->type = tok_keyword;
+        }
 
       return n;
     }
@@ -717,16 +777,31 @@ parser::parse ()
 	    break;
 
           empty = false;
-	  if (t->type == tok_identifier && t->content == "probe")
-            parse_probe (f->probes, f->aliases);
-	  else if (t->type == tok_identifier && t->content == "global")
-	    parse_global (f->globals);
-	  else if (t->type == tok_identifier && t->content == "function")
-            parse_functiondecl (f->functions);
+	  if (t->type == tok_keyword && t->content == "probe")
+	    {
+	      context = con_probe;
+	      parse_probe (f->probes, f->aliases);
+	    }
+	  else if (t->type == tok_keyword && t->content == "global")
+	    {
+	      context = con_global;
+	      parse_global (f->globals);
+	    }
+	  else if (t->type == tok_keyword && t->content == "function")
+	    {
+	      context = con_function;
+	      parse_functiondecl (f->functions);
+	    }
           else if (t->type == tok_embedded)
-            f->embeds.push_back (parse_embeddedcode ());
+	    {
+	      context = con_embedded;
+	      f->embeds.push_back (parse_embeddedcode ());
+	    }
 	  else
-	    throw parse_error ("expected 'probe', 'global', 'function', or '%{'");
+	    {
+	      context = con_unknown;
+	      throw parse_error ("expected 'probe', 'global', 'function', or '%{'");
+	    }
 	}
       catch (parse_error& pe)
 	{
@@ -774,7 +849,7 @@ parser::parse_probe (std::vector<probe *> & probe_ret,
 		     std::vector<probe_alias *> & alias_ret)
 {
   const token* t0 = next ();
-  if (! (t0->type == tok_identifier && t0->content == "probe"))
+  if (! (t0->type == tok_keyword && t0->content == "probe"))
     throw parse_error ("expected 'probe'");
 
   vector<probe_point *> aliases;
@@ -782,6 +857,8 @@ parser::parse_probe (std::vector<probe *> & probe_ret,
 
   bool equals_ok = true;
 
+  int epilogue_alias = 0;
+
   while (1)
     {
       probe_point * pp = parse_probe_point ();
@@ -794,6 +871,15 @@ parser::parse_probe (std::vector<probe *> & probe_ret,
           next ();
           continue;
         }
+      else if (equals_ok && t 
+          && t->type == tok_operator && t->content == "+=")
+        {
+          aliases.push_back(pp);
+          epilogue_alias = 1;
+          next ();
+          continue;
+        }
+
       else if (t && t->type == tok_operator && t->content == ",")
         {
           locations.push_back(pp);
@@ -821,6 +907,10 @@ parser::parse_probe (std::vector<probe *> & probe_ret,
   else
     {
       probe_alias* p = new probe_alias (aliases);
+      if(epilogue_alias)
+	p->epilogue_style = true;
+      else
+	p->epilogue_style = false;
       p->tok = t0;
       p->locations = locations;
       p->body = parse_stmt_block ();
@@ -903,23 +993,23 @@ parser::parse_statement ()
     }
   else if (t && t->type == tok_operator && t->content == "{")  
     return parse_stmt_block ();
-  else if (t && t->type == tok_identifier && t->content == "if")
+  else if (t && t->type == tok_keyword && t->content == "if")
     return parse_if_statement ();
-  else if (t && t->type == tok_identifier && t->content == "for")
+  else if (t && t->type == tok_keyword && t->content == "for")
     return parse_for_loop ();
-  else if (t && t->type == tok_identifier && t->content == "foreach")
+  else if (t && t->type == tok_keyword && t->content == "foreach")
     return parse_foreach_loop ();
-  else if (t && t->type == tok_identifier && t->content == "return")
+  else if (t && t->type == tok_keyword && t->content == "return")
     return parse_return_statement ();
-  else if (t && t->type == tok_identifier && t->content == "delete")
+  else if (t && t->type == tok_keyword && t->content == "delete")
     return parse_delete_statement ();
-  else if (t && t->type == tok_identifier && t->content == "while")
+  else if (t && t->type == tok_keyword && t->content == "while")
     return parse_while_loop ();
-  else if (t && t->type == tok_identifier && t->content == "break")
+  else if (t && t->type == tok_keyword && t->content == "break")
     return parse_break_statement ();
-  else if (t && t->type == tok_identifier && t->content == "continue")
+  else if (t && t->type == tok_keyword && t->content == "continue")
     return parse_continue_statement ();
-  else if (t && t->type == tok_identifier && t->content == "next")
+  else if (t && t->type == tok_keyword && t->content == "next")
     return parse_next_statement ();
   // XXX: "do/while" statement?
   else if (t && (t->type == tok_operator || // expressions are flexible
@@ -937,7 +1027,7 @@ void
 parser::parse_global (vector <vardecl*>& globals)
 {
   const token* t0 = next ();
-  if (! (t0->type == tok_identifier && t0->content == "global"))
+  if (! (t0->type == tok_keyword && t0->content == "global"))
     throw parse_error ("expected 'global'");
 
   while (1)
@@ -971,12 +1061,14 @@ void
 parser::parse_functiondecl (std::vector<functiondecl*>& functions)
 {
   const token* t = next ();
-  if (! (t->type == tok_identifier && t->content == "function"))
+  if (! (t->type == tok_keyword && t->content == "function"))
     throw parse_error ("expected 'function'");
 
 
   t = next ();
-  if (! (t->type == tok_identifier))
+  if (! (t->type == tok_identifier)
+      && ! (t->type == tok_keyword
+	    && (t->content == "string" || t->content == "long")))
     throw parse_error ("expected identifier");
 
   for (unsigned i=0; i<functions.size(); i++)
@@ -991,9 +1083,9 @@ parser::parse_functiondecl (std::vector<functiondecl*>& functions)
   if (t->type == tok_operator && t->content == ":")
     {
       t = next ();
-      if (t->type == tok_identifier && t->content == "string")
+      if (t->type == tok_keyword && t->content == "string")
 	fd->type = pe_string;
-      else if (t->type == tok_identifier && t->content == "long")
+      else if (t->type == tok_keyword && t->content == "long")
 	fd->type = pe_long;
       else throw parse_error ("expected 'string' or 'long'");
 
@@ -1021,9 +1113,9 @@ parser::parse_functiondecl (std::vector<functiondecl*>& functions)
       if (t->type == tok_operator && t->content == ":")
 	{
 	  t = next ();
-	  if (t->type == tok_identifier && t->content == "string")
+	  if (t->type == tok_keyword && t->content == "string")
 	    vd->type = pe_string;
-	  else if (t->type == tok_identifier && t->content == "long")
+	  else if (t->type == tok_keyword && t->content == "long")
 	    vd->type = pe_long;
 	  else throw parse_error ("expected 'string' or 'long'");
 	  
@@ -1055,8 +1147,10 @@ parser::parse_probe_point ()
   while (1)
     {
       const token* t = next ();
-      if (! (t->type == tok_identifier ||
-             (t->type == tok_operator && t->content == "*")))
+      if (! (t->type == tok_identifier
+	     // we must allow ".return" and ".function", which are keywords
+	     || t->type == tok_keyword
+	     || (t->type == tok_operator && t->content == "*")))
         throw parse_error ("expected identifier or '*'");
 
       if (pl->tok == 0) pl->tok = t;
@@ -1064,13 +1158,31 @@ parser::parse_probe_point ()
       probe_point::component* c = new probe_point::component;
       c->functor = t->content;
       pl->components.push_back (c);
-      // NB though we still may add c->arg soon
+      // NB we may add c->arg soon
 
+      const token* last_t = t;
       t = peek ();
-      if (t && t->type == tok_operator 
-          && (t->content == "{" || t->content == "," || t->content == "="))
-        break;
-      
+
+      // We need to keep going until we find something other than a
+      // '*' or identifier, since a probe point wildcard can be
+      // something like "*a", "*a*", "a*b", "a*b*", etc.
+      while (t &&
+	     // case 1: '*{identifier}'
+	     ((last_t->type == tok_operator && last_t->content == "*"
+	       && (t->type == tok_identifier || t->type == tok_keyword))
+	      // case 2: '{identifier}*'
+	      || ((last_t->type == tok_identifier
+		   || last_t->type == tok_keyword)
+		  && t->type == tok_operator && t->content == "*")))
+        {
+	  c->functor += t->content;
+	  next ();			// consume the identifier or '*'
+
+	  last_t = t;
+	  t = peek ();
+	}
+
+      // consume optional parameter
       if (t && t->type == tok_operator && t->content == "(")
         {
           next (); // consume "("
@@ -1081,19 +1193,31 @@ parser::parse_probe_point ()
             throw parse_error ("expected ')'");
 
           t = peek ();
-          if (t && t->type == tok_operator 
-              && (t->content == "{" || t->content == "," || t->content == "="))
-            break;
-          else if (t && t->type == tok_operator &&
-                   t->content == "(")
-            throw parse_error ("unexpected '.' or ',' or '{'");
         }
-      // fall through
 
       if (t && t->type == tok_operator && t->content == ".")
-        next ();
-      else
-        throw parse_error ("expected '.' or ',' or '(' or '{' or '='");
+        {
+          next ();
+          continue;
+        }
+
+      // We only fall through here at the end of a probe point (past
+      // all the dotted/parametrized components).
+
+      if (t && t->type == tok_operator && t->content == "?")
+        {
+          pl->optional = true;
+          next ();
+          t = peek ();
+          // fall through
+        }
+
+      if (t && t->type == tok_operator 
+          && (t->content == "{" || t->content == "," ||
+              t->content == "=" || t->content == "+=" ))
+        break;
+      
+      throw parse_error ("expected '.' or ',' or '(' or '?' or '{' or '=' or '+='");
     }
 
   return pl;
@@ -1136,7 +1260,7 @@ if_statement*
 parser::parse_if_statement ()
 {
   const token* t = next ();
-  if (! (t->type == tok_identifier && t->content == "if"))
+  if (! (t->type == tok_keyword && t->content == "if"))
     throw parse_error ("expected 'if'");
   if_statement* s = new if_statement;
   s->tok = t;
@@ -1154,7 +1278,7 @@ parser::parse_if_statement ()
   s->thenblock = parse_statement ();
 
   t = peek ();
-  if (t && t->type == tok_identifier && t->content == "else")
+  if (t && t->type == tok_keyword && t->content == "else")
     {
       next ();
       s->elseblock = parse_statement ();
@@ -1181,8 +1305,10 @@ return_statement*
 parser::parse_return_statement ()
 {
   const token* t = next ();
-  if (! (t->type == tok_identifier && t->content == "return"))
+  if (! (t->type == tok_keyword && t->content == "return"))
     throw parse_error ("expected 'return'");
+  if (context != con_function)
+    throw parse_error ("found 'return' not in function context");
   return_statement* s = new return_statement;
   s->tok = t;
   s->value = parse_expression ();
@@ -1194,7 +1320,7 @@ delete_statement*
 parser::parse_delete_statement ()
 {
   const token* t = next ();
-  if (! (t->type == tok_identifier && t->content == "delete"))
+  if (! (t->type == tok_keyword && t->content == "delete"))
     throw parse_error ("expected 'delete'");
   delete_statement* s = new delete_statement;
   s->tok = t;
@@ -1207,8 +1333,10 @@ next_statement*
 parser::parse_next_statement ()
 {
   const token* t = next ();
-  if (! (t->type == tok_identifier && t->content == "next"))
+  if (! (t->type == tok_keyword && t->content == "next"))
     throw parse_error ("expected 'next'");
+  if (context != con_probe)
+    throw parse_error ("found 'next' not in probe context");
   next_statement* s = new next_statement;
   s->tok = t;
   return s;
@@ -1219,7 +1347,7 @@ break_statement*
 parser::parse_break_statement ()
 {
   const token* t = next ();
-  if (! (t->type == tok_identifier && t->content == "break"))
+  if (! (t->type == tok_keyword && t->content == "break"))
     throw parse_error ("expected 'break'");
   break_statement* s = new break_statement;
   s->tok = t;
@@ -1231,7 +1359,7 @@ continue_statement*
 parser::parse_continue_statement ()
 {
   const token* t = next ();
-  if (! (t->type == tok_identifier && t->content == "continue"))
+  if (! (t->type == tok_keyword && t->content == "continue"))
     throw parse_error ("expected 'continue'");
   continue_statement* s = new continue_statement;
   s->tok = t;
@@ -1243,7 +1371,7 @@ for_loop*
 parser::parse_for_loop ()
 {
   const token* t = next ();
-  if (! (t->type == tok_identifier && t->content == "for"))
+  if (! (t->type == tok_keyword && t->content == "for"))
     throw parse_error ("expected 'for'");
   for_loop* s = new for_loop;
   s->tok = t;
@@ -1295,7 +1423,7 @@ parser::parse_for_loop ()
       s->incr = parse_expr_statement ();
       t = next ();
       if (! (t->type == tok_operator && t->content == ")"))
-	throw parse_error ("expected ';'");
+	throw parse_error ("expected ')'");
     }
 
   // block
@@ -1309,7 +1437,7 @@ for_loop*
 parser::parse_while_loop ()
 {
   const token* t = next ();
-  if (! (t->type == tok_identifier && t->content == "while"))
+  if (! (t->type == tok_keyword && t->content == "while"))
     throw parse_error ("expected 'while'");
   for_loop* s = new for_loop;
   s->tok = t;
@@ -1340,7 +1468,7 @@ foreach_loop*
 parser::parse_foreach_loop ()
 {
   const token* t = next ();
-  if (! (t->type == tok_identifier && t->content == "foreach"))
+  if (! (t->type == tok_keyword && t->content == "foreach"))
     throw parse_error ("expected 'foreach'");
   foreach_loop* s = new foreach_loop;
   s->tok = t;
@@ -1402,7 +1530,7 @@ parser::parse_foreach_loop ()
     }
 
   t = next ();
-  if (! (t->type == tok_identifier && t->content == "in"))
+  if (! (t->type == tok_keyword && t->content == "in"))
     throw parse_error ("expected 'in'");
  
   s->base = parse_indexable();
@@ -1648,7 +1776,7 @@ parser::parse_array_in ()
     }
 
   t = peek ();
-  if (t && t->type == tok_identifier && t->content == "in")
+  if (t && t->type == tok_keyword && t->content == "in")
     {
       array_in *e = new array_in;
       e->tok = t;
@@ -1986,6 +2114,7 @@ parser::parse_symbol ()
 	  fmt->tok = t;
 	  fmt->print_with_format = (name[name.size() - 1] == 'f');
 	  fmt->print_to_stream = (name[0] == 'p');
+
 	  expect_op("(");
 	  if (fmt->print_with_format)
 	    {
@@ -2061,7 +2190,7 @@ parser::parse_symbol ()
 	      if (peek_op ("->"))
 		{ 
 		  next(); 
-		  expect_ident (c);
+		  expect_ident_or_keyword (c);
 		  tsym->components.push_back
 		    (make_pair (target_symbol::comp_struct_member, c));
 		}
@@ -2121,7 +2250,7 @@ parser::parse_symbol ()
   // By now, either we had a hist_op in the first place, or else 
   // we had a plain word and it was converted to a symbol.
 
-  assert (hop || sym);
+  assert (!hop != !sym); // logical XOR
 
   // All that remains is to check for array indexing
 
