@@ -24,12 +24,18 @@ struct source_loc
   unsigned column;
 };
 
+std::ostream& operator << (std::ostream& o, const source_loc& loc);
+
+enum parse_context
+  {
+    con_unknown, con_probe, con_global, con_function, con_embedded
+  };
+
 
 enum token_type 
   {
     tok_junk, tok_identifier, tok_operator, tok_string, tok_number,
-    tok_embedded
-    // XXX: add tok_keyword throughout
+    tok_embedded, tok_keyword
   };
 
 
@@ -89,6 +95,7 @@ private:
   std::istream* free_input;
   lexer input;
   bool privileged;
+  parse_context context;
 
   // preprocessing subordinate
   std::vector<const token*> enqueued_pp;
@@ -105,12 +112,15 @@ private:
   // expectations
   const token* expect_known (token_type tt, std::string const & expected);
   const token* expect_unknown (token_type tt, std::string & target);
+  const token* expect_unknown2 (token_type tt1, token_type tt2,
+				std::string & target);
 
   // convenience forms
   const token* expect_op (std::string const & expected);
   const token* expect_kw (std::string const & expected);
   const token* expect_number (int64_t & expected);
   const token* expect_ident (std::string & target);
+  const token* expect_ident_or_keyword (std::string & target);
   bool peek_op (std::string const & op);
   bool peek_kw (std::string const & kw);
 
